__title__ = "sgtpyutils"
__description__ = "sgtpyutils is a public library for use."
__keywords__ = ['lib', 'utils']
__url__ = "https://github.com/serfend/sgtpyutils"
__version__ = "1.15.10"
__author__ = "serfend"
__author_email__ = "serfend@foxmail.com"
__license__ = "MIT Licence"

__public_path__ = './dist'
