from typing import List, Type, overload
from .node import Node
from .base_tree import BinaryTree as BaseTree
from .full_tree import BinaryTree as FullTree
from .full_tree_ex import BinaryTree as FullTreeEx
from .ordered_tree import BinaryTree as OrderedTree
BinaryTree = OrderedTree
